/*
******************************************************************************

   LibBuf extension internals.

   Copyright (c) Wed Aug 29 2001 by: 
	Brian S. Julin		bri@tull.umassp.edu
        Christoph Egger         Christoph_Egger@t-online.de
  
   
This software may be redistributed under the terms of the GNU Library
(a.k.a. "Lesser") Public License -- see the file COPYING in the parent 
directory of the source code.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


******************************************************************************
*/

#ifndef _GGI_BUF_INT_H
#define _GGI_BUF_INT_H

#include <ggi/internal/internal.h>

#define HAVE_STRUCT_BUF
#include <ggi/buf.h>

__BEGIN_DECLS
/* Exported variables */
#ifdef BUILDING_LIBBUF
extern ggi_extid    _BufID;
#else
IMPORTVAR ggi_extid _BufID;
#endif

__END_DECLS

/* Our lovely getapi suggest string munger */
#define BUF_GETAPI_GIVE_PIXFMT 1
#define BUF_GETAPI_ALPHA_USED  2
void BUF_getapi_fstr (ggi_visual *vis, char* str, int flags);

struct _buf_def_llobj {
  GGI_LL_OBJ(6)
  struct ggi_ll_stops	stops;
  struct ggi_ll_ststate	ststate;
  struct ggi_ll_fgbg_gc	gc, gc2;
  struct ggi_ll_math_gc	mathgc;
  ggiGA_resource_handle ga[4];
};

/* Internal functions provided in alloc.c */
ggiBuf_t _buf_alloc_def_llobj(void);
void _buf_free_def_llobj(ggiBuf_t buf);

/* This struct should never be alloced or sizeof() taken.   It is 
 * defined for the sake of bounds checkers.  Well, OK, to deceive bounds
 * checkers really.  :-) 
 */
struct ggiBuf {
        GGI_LL_OBJ(16)
};

/* These are per-visual overloads but not visible in public API */
int ggiBuf_untie(ggi_visual *vis, ggiBuf_t *buf);
int ggiBuf_clad(ggi_visual *vis, int flags, int numbufs, 
		ggiBuf_t *bufs, ggiBuf_t *res);

/* Values for ggiBuf_clad's flags parameter */
#define BUF_CLAD_VISUAL		0x0001

/* Exported variables */
#ifdef BUILDING_LIBBUF
extern ggi_extid    _BufID;
#else
IMPORTVAR ggi_extid _BufID;
#endif

__END_DECLS

/*-* Data types for the API functions */
typedef int (Buffunc_ggiBufStrip)(ggi_visual *vis);
typedef int (Buffunc_ggiBuf_untie)(ggi_visual *vis,
				   ggiBuf_t *buf, 
				   ggiGA_resource_handle carb_h,
				   ggiGA_resource_handle tank_h
				   );
typedef int (Buffunc_ggiBuf_clad)(ggi_visual *vis,
				  int flags, int numbufs,
				  ggiBuf_t *bufs, ggiBuf_t *res);
typedef ggiBuf_t (Buffunc_ggiBuf_tie)(ggi_visual_t vis, 
				      ggiGA_resource_handle motor_h,
				      ggiGA_resource_handle carb_h, 
				      ggiGA_resource_handle tank_h,
				      int narg, void *args);

typedef struct bufpriv {
	/*-* API function pointers */
	Buffunc_ggiBufStrip	*strip;
	Buffunc_ggiBuf_untie	*_untie;
	Buffunc_ggiBuf_clad	*_clad;
	Buffunc_ggiBuf_tie	*_tie;


	/* pointer is needed to override libggi's getapi function */
	ggifunc_getapi		*getapi;
	void 			*cfghand; /* Hack for storage sublibs */

	struct ggi_ll_stops def_ops;

	/*-* Other structure members */
	int is_clad;
	struct ggi_visual_opdraw * old_opdraw;

	/* We are clad with this Alpha buffer */
	ggiBuf_t abuf;	
	int achannel;			/* In-channel Alpha if true  */
	struct ggi_ll_fgbg_gc agc;      /* hw state shadowed here.   */
	struct ggi_ll_fgbg_gc agc2;     /* hw state shadowed here.   */
	struct ggi_ll_math_gc amathgc;  /* hw state shadowed here.   */

	/* We are clad with this Z buffer */
	ggiBuf_t zbuf;
	struct ggi_ll_fgbg_gc zgc;      /* hw state shadowed here. */
	struct ggi_ll_fgbg_gc zgc2;     /* hw state shadowed here. */
	struct ggi_ll_math_gc zmathgc;  /* hw state shadowed here. */

	ggiGA_resource_list *reqlist;  /* Pointer to application provided
                                          "global" request list for lazy API
                                        */

	void *priv;	/* Here sublibs can store visual specific private data.
			 * If you have more than one sublib (for each visual)
			 * that needs private data, just add more pointers or
			 * add the data fields directly in this structure.
			 */
} bufpriv;


/* Used to access the bufpriv structure for a particular visual. */
#define LIBGGI_BUFEXT(vis) ((bufpriv *)LIBGGI_EXT((vis),_BufID))

#endif /* _GGI_BUF_INT_H */
