/*
******************************************************************************

   LibOVL: plain functions

   Copyright (C) 2001 Christoph Egger   [Christoph_Egger@t-online.de]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <ggi/internal/ovl.h>
#include <ggi/internal/ovl_debug.h>
#include <ggi/internal/galloc_ops.h>



ggiGA_resource_handle ggiOvlGetMotorHandle(ggiOvl_t ovl)
{
	if (ovl == NULL) return NULL;

	return ovl->res_handle_motor;
}	/* ggiOvlGetMotorHandle */


ggiGA_resource_handle ggiOvlGetTankHandle(ggiOvl_t ovl)
{
	if (ovl == NULL) return NULL;

	return ovl->res_handle_tank;
}	/* ggiOvlGetTankHandle */


ggiGA_resource_handle ggiOvlGetCarbHandle(ggiOvl_t ovl)
{
	if (ovl == NULL) return NULL;

	return ovl->res_handle_carb;
}	/* ggiOvlGetCarbHandle */




/* Fills properties with useful default values */
int ggiOvlClearMotorProperties(struct ggiGA_resource_props *prop)
{
	int rc = GALLOC_OK;
	LIB_ASSERT(prop != NULL);

	rc = ggiGAClearMotorProperties(prop);
	LIB_ASSERT(rc == GALLOC_OK);

	return rc;
}	/* ggiOvlClearMotorProperties */


int ggiOvlClearCarbProperties(struct ggiGA_resource_props *prop)
{
	int rc = GALLOC_OK;
	LIB_ASSERT(prop != NULL);

	rc = ggiGAClearCarbProperties(prop);
	LIB_ASSERT(rc == GALLOC_OK);

	return rc;
}	/* ggiOvlClearCarbProperties */


int ggiOvlClearTankProperties(struct ggiGA_resource_props *prop)
{
	int rc = GALLOC_OK;
	LIB_ASSERT(prop != NULL);

	rc = ggiGAClearTankProperties(prop);
	LIB_ASSERT(rc == GALLOC_OK);

	return rc;
}	/* ggiOvlClearTankProperties */




int ggiOvlSetCoordBase(ggiOvl_t ovl, unsigned int val)
{
	uint16 tmp;

	tmp = (val & LL_COORDBASE_MASK);
	if (tmp == LL_COORDBASE_PIXELS || tmp == LL_COORDBASE_DOTS) {
		tmp |= (val & GA_FLAG_UNIT_MASK);
	}	/* if */

	if (tmp != val) return GALLOC_EFAILED;

	ovl->user_cb = tmp;

	return GALLOC_OK;
}	/* ggiOvlSetCoordBase */




int ggiOvlGetPos(ggiOvl_t ovl, long *x, long *y)
{
	*x = ovl->cur_x;
	*y = ovl->cur_y;

#warning convert *x and *y to the unit set in ovl->user_cb

	return GALLOC_OK;
}	/* ggiOvlGetPos */


int ggiOvlGetSize(ggiOvl_t ovl, long *w, long *h)
{
	LIB_ASSERT(ovl != NULL);

	*w = ovl->cur_w;
	*h = ovl->cur_h;

#warning convert *w and *h to the unit set in ovl->user_cb

	return GALLOC_EFAILED;
}	/* ggiOvlGetSize */


int ggiOvlGetZoom(ggiOvl_t ovl, long *x, long *y, long *w, long *h)
{
	LIB_ASSERT(ovl != NULL);

	*x = ovl->cur_x;
	*y = ovl->cur_y;

	*w = ovl->cur_w;
	*h = ovl->cur_h;

#warning convert *x, *y, *w and *h to the unit set in ovl->user_cb

	return GALLOC_EFAILED;
}	/* ggiOvlGetZoom */


int ggiOvlGetPriority(ggiOvl_t ovl, uint32 *slot)
{
	LIB_ASSERT(ovl != NULL);
	LIB_ASSERT(slot != NULL);

	*slot = ovl->slot;

	return GALLOC_OK;
}	/* ggiOvlGetPriority */


enum ggiOvl_flags ggiOvlGetFlags(ggiOvl_t ovl)
{
	LIB_ASSERT(ovl != NULL);

	return (enum ggiOvl_flags)(ovl->flags);
}	/* ggiOvlGetFlags */



int _ggiOvlSync(ggiOvl_t ovl)
{
	LIB_ASSERT(ovl != NULL);

	if (!ovl->zoomed) {

		ovl->x = ovl->cur_x;
		ovl->y = ovl->cur_y;

		ovl->w = ovl->cur_w;
		ovl->h = ovl->cur_h;

	}	/* if */

	return GALLOC_OK;
}	/* _ggiOvlSync */
